#region imports
import inspect

from delog.constants import (
    CALL_CONTEXT,
    REPOSITORY_PROVIDER,
    REPOSITORY_NAME,
    REPOSITORY_BRANCH,
    REPOSITORY_COMMIT,
    REPOSITORY_BASEPATH,
)
#endregion imports



#region module
def get_caller(
    call: dict = {},
):
    if not bool(call) and not CALL_CONTEXT:
        return None

    if not call:
        call = {}

    call_repository = call.get("repository", {})

    if not call_repository:
        return None

    provider = call_repository.get("provider", REPOSITORY_PROVIDER)
    repository_name = call_repository.get("name", REPOSITORY_NAME)
    repository_branch = call_repository.get("branch", REPOSITORY_BRANCH)
    repository_commit = call_repository.get("commit", REPOSITORY_COMMIT)
    repository_basepath = call_repository.get("basepath", REPOSITORY_BASEPATH)

    stack = inspect.stack()

    file = stack[2][1]
    line = int(stack[2][2])

    filepath = file.replace(repository_basepath, "", 1)

    caller = {
        "repository": {
            "provider": provider,
            "name": repository_name,
            "branch": repository_branch,
            "commit": repository_commit,
            "basePath": repository_basepath,
        },
        "caller": {
            "file": filepath,
            "line": line,
            "column": 0,
        },
    }

    return caller
#endregion module



#region imports
import os
#endregion imports



#region module
QUIET = os.environ.get('DELOG_QUIET', '')

GROUND_LEVEL = int(os.environ.get('DELOG_GROUND_LEVEL', 0))

FORMAT = os.environ.get('DELOG_FORMAT', '')

ENDPOINT = os.environ.get('DELOG_ENDPOINT')
TOKEN = os.environ.get('DELOG_TOKEN')

PROJECT = os.environ.get('DELOG_PROJECT', '')
SPACE = os.environ.get('DELOG_SPACE', '')

CALL_CONTEXT = bool(os.environ.get('DELOG_CALL_CONTEXT', False))
REPOSITORY_PROVIDER = os.environ.get("DELOG_REPOSITORY_PROVIDER", '')
REPOSITORY_NAME = os.environ.get("DELOG_REPOSITORY_NAME", '')
REPOSITORY_BRANCH = os.environ.get("DELOG_REPOSITORY_BRANCH", 'master')
REPOSITORY_COMMIT = os.environ.get("DELOG_REPOSITORY_COMMIT", 'latest')
REPOSITORY_BASEPATH = os.environ.get("DELOG_REPOSITORY_BASEPATH", '__MATCH_CUT__')


DELOG_LEVEL_FATAL = 6
DELOG_LEVEL_ERROR = 5
DELOG_LEVEL_WARN = 4
DELOG_LEVEL_INFO = 3
DELOG_LEVEL_DEBUG = 2
DELOG_LEVEL_TRACE = 1

delog_levels = {
    "fatal": DELOG_LEVEL_FATAL,
    "error": DELOG_LEVEL_ERROR,
    "warn": DELOG_LEVEL_WARN,
    "info": DELOG_LEVEL_INFO,
    "debug": DELOG_LEVEL_DEBUG,
    "trace": DELOG_LEVEL_TRACE,
}
#endregion module



#region imports
from python_graphql_client import GraphqlClient

from delog.constants import (
    ENDPOINT,
    TOKEN,
)
#endregion imports



#region module
RECORD = """
    mutation DelogMutationRecord($input: DelogInputRecord!) {
        delogMutationRecord(input: $input) {
            status
        }
    }
"""


def client(
    endpoint: str = ENDPOINT,
    token: str = TOKEN,
):
    token_value = 'Bearer ' + token

    headers = {
        "Authorization": token_value
    }

    client = GraphqlClient(
        endpoint=endpoint,
        headers=headers,
    )

    return client
#endregion module



#region exports
from .delog import delog

from .constants import (
    delog_levels,
    DELOG_LEVEL_FATAL,
    DELOG_LEVEL_ERROR,
    DELOG_LEVEL_WARN,
    DELOG_LEVEL_INFO,
    DELOG_LEVEL_DEBUG,
    DELOG_LEVEL_TRACE,
)
#endregion exports



import setuptools



with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="delog",
    version="0.0.8",
    author="ly3xqhl8g9",
    author_email="ly3xqhl8g9@plurid.com",
    description="client for centralized logging on a delog server",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/plurid/delog",
    packages=setuptools.find_packages(exclude=("tests",)),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'python-graphql-client',
    ],
)



#region imports
import time

from delog.graphql import (
    RECORD,
    client,
)

from delog.constants import (
    GROUND_LEVEL,
    FORMAT,
    ENDPOINT,
    TOKEN,
    PROJECT,
    SPACE,
    delog_levels,
)

from delog.caller import (
    get_caller,
)

from delog.utilities import (
    console_log,
)
#endregion imports



#region module
def delog(
    text: str,

    #  Log level:
    #  + FATAL: 6;
    #  + ERROR: 5;
    #  + WARN: 4;
    #  + INFO: 3;
    #  + DEBUG: 2;
    #  + TRACE: 1;
    level: int = delog_levels["info"],

    endpoint: str = ENDPOINT,
    token: str = TOKEN,

    format: str = FORMAT,

    project: str = PROJECT,
    space: str = SPACE,

    # To be used if the `delog` is meant to be fired only in 'TESTING' `mode` (`context.mode`),
    # and the `mode` is set dinamically/from outside the enclosing function.
    tester: bool = False,

    # Name of the method from where the log originates.
    method: str = None,

    error = None,

    # Arbitrary data: a simple string, stringified JSON or deon.
    extradata: str = None,

    context: dict = {},
):
    if not context:
        context = {}

    if not endpoint:
        console_log("Delog Error :: An endpoint is required.")
        return

    if not token:
        console_log("Delog Error :: A token is required.")
        return

    if tester and context.get("mode") != "TESTING":
        return

    if GROUND_LEVEL > level:
        return

    try:
        graphql_client = client(
            endpoint=endpoint,
            token=token,
        )

        call_context = get_caller(
            call=context.get("call"),
        )

        log_time = int(time.time())
        error_string = repr(error)
        input_context = {
            "mode": context.get("mode", "LOGGING"),
            "scenario": context.get("scenario", None),
            "suite": context.get("suite", None),
            "sharedID": context.get("shared_id", None),
            "sharedOrder": context.get("shared_order", None),
            "call": call_context,
        };

        variables = {
            "input": {
                "text": text,
                "time": log_time,
                "level": level,

                "project": project,
                "space": space,

                "format": format,

                "method": method,
                "error": error_string,
                "extradata": extradata,
                "context": input_context,
            }
        }

        graphql_client.execute(
            query=RECORD,
            variables=variables,
        )
    except:
        console_log("Delog Error :: Something went wrong.")
#endregion module
